use serde::{Serialize, Deserialize};

use warp::{reject::Rejection, reply::{json, with_status, Reply}, Filter, http::StatusCode};

use crate::{executor::{executors, executors_id, get_executor, python::PythonExecutor, Executor}, SharedState};

#[derive(Serialize, Deserialize)]
struct ExecutorData {
    code: String,
    lang: String
}

async fn executor(_state: SharedState, data: ExecutorData) -> Result<Box<dyn Reply>, Rejection> {
    
    let lang = data.lang;
    let execs = executors();
    let found = get_executor(&lang, &execs);
    
    if found.is_none() {
        return Ok(
            Box::new(
                with_status(
                    json(&format!("{lang} is not supported")),
                    StatusCode::BAD_REQUEST
                )
            )
        )
    }

    let lang = found.unwrap();
    let out = lang.exec(data.code).unwrap();
    
    Ok(Box::new(warp::reply::json(&out)))
}

fn executor_f(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    warp::post()
        .map(move || state.clone())
        .and(warp::body::json())
        .and_then(executor)
}

pub fn routes(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    executor_f(state)
}